!****h* Attitude/AsceDecl_class
! NAME
! AsceDecl_class -- Calculates Right Ascension and Declination
! FUNCTION
! Contains functions to calculate the Right Ascension and Declination
! of a non null tridimensional vector "vect" in the inertial equatorial coordinate system
! AUTHOR
! Paolo Zoccarato
! CREATION DATE
! 2006/06/20
! MODIFICATION HISTORY
! +------------------------------------------------------------------+
! |  name      |  date  |                  reason                    |
! |------------|--------|--------------------------------------------|
! |            |        |  No one at the moment                      |
! +------------|--------|--------------------------------------------+
! |            |        |                                            |
! +------------------------------------------------------------------+
! COMMANDS
! PUBLIC FUNCTIONS:
!	AR(vect) -- Calculates the Right Ascension of the vector "vect"
!		INPUT PARAMETERS:
!			vect(3) : non null tridimensional vector in the inertial equatorial coordinate system
!		OUTPUT PARAMETERS:
!			AR      : Right Ascension of the vector "vect" in the inertial equatorial coordinate system
!
!	Dec(vect) -- Calculates the Declination of the vector "vect"
!		INPUT PARAMETERS:
!			vect(3) : non null tridimensional vector in the inertial equatorial coordinate system
!		OUTPUT PARAMETERS:
!			Dec     : Right Ascension of the vector "vect" in the inertial equatorial coordinate system
! USES
! This module uses the parameters "i, j, k, norm2" of the module "Globals_class"
! NOTES
! Develloped with Compaq Visual Fortran for Windows on Windows XP
! SOURCE

MODULE AsceDecl_class

USE Globals_class, ONLY : norm2, rad2deg, deg2rad, pi

! Modulo che contiene le funzioni per il calcolo dell'ascensione retta e della declinazione
!
! PUBLIC FUNCTIONS:
! AR(vect)
! Dec(vect)

IMPLICIT NONE

CONTAINS

! ============================================================
REAL(8) FUNCTION AR(vect)

! Funzione che dato un vettore ne calcola l'ascensione retta rispetto al sdr inerziale equatoriale
!
! PARAMETRI DI INPUT:
! vect : vettore tridimensionale non nullo
!
! PARAMETRI DI OUTPUT:
! AR   : ascensione retta del vettore vect rispetto al sdr inerziale equatoriale

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: vect(3)

! Inizio procedimento
IF ((vect(1)==0.0).AND.(vect(2)==0.0).AND.(vect(3)==0.0)) THEN
	WRITE(6,*) 'Error, null vector in input'
	STOP
END IF

AR = DATAN2(DOT_PRODUCT((/ 0.d0, 1.d0, 0.d0 /), vect/norm2(vect)), DOT_PRODUCT((/ 1.d0, 0.d0, 0.d0 /), vect/norm2(vect)))

AR = rad2deg(AR)

END FUNCTION AR
! ============================================================

! ============================================================
REAL(8) FUNCTION Dec(vect)

! Funzione che dato un vettore ne calcola la declinazione rispetto al sdr inerziale equatoriale
!
! PARAMETRI DI INPUT:
! vect : vettore tridimensionale non nullo
!
! PARAMETRI DI OUTPUT:
! Dec  : declinazione del vettore vect rispetto al sdr inerziale equatoriale

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: vect(3)

! Inizio procedimento
IF ((vect(1)==0.0).AND.(vect(2)==0.0).AND.(vect(3)==0.0)) THEN
	WRITE(6,*) 'Error, null vector in input'
	STOP
END IF

DEC = DATAN2(DOT_PRODUCT((/ 0.d0, 0.d0, 1.d0 /), vect/norm2(vect)), SQRT((DOT_PRODUCT((/ 1.d0, 0.d0, 0.d0 /), vect/norm2(vect)))**2+(DOT_PRODUCT((/ 0.d0, 1.d0, 0.d0 /), vect/norm2(vect)))**2))

DEC = rad2deg(DEC)

END FUNCTION Dec
! ============================================================

! ============================================================
REAL(8) FUNCTION Eps(asce, decl)

! Funzione che dati
!
! PARAMETRI DI INPUT:
! 
!
! PARAMETRI DI OUTPUT:
! Eps:

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: asce, decl
REAL(8) :: ascerad, declrad

! Inizio procedimento
ascerad = deg2rad(asce)
declrad = deg2rad(decl)
IF ((asce>0d0).AND.(asce<=90d0)) THEN
	Eps = DATAN(DTAN(declrad)/DSIN(ascerad))
ELSEIF (asce>90d0) THEN
    Eps = DATAN(DTAN(declrad)/DSIN(ascerad-pi/2))
ELSEIF ((asce<0d0).AND.(asce>=-90d0))THEN
    Eps = DATAN(DTAN(DABS(declrad))/DSIN(DABS(ascerad)))
ELSEIF (asce<-90d0) THEN
    Eps = DATAN(DTAN(DABS(declrad))/DSIN(DABS(ascerad+pi/2)))
END IF

Eps = rad2deg(Eps)

END FUNCTION Eps
! ============================================================

! ============================================================
REAL(8) FUNCTION Decli(asce, eps)

! Funzione che dati
!
! PARAMETRI DI INPUT:
! 
!
! PARAMETRI DI OUTPUT:
! Eps:

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: asce, eps
REAL(8) :: asce2, asce2rad, epsrad

! Inizio procedimento
asce2 = asce-pi/2d0;
IF (asce2<-180d0) THEN
    asce2 = 180d0-(asce2+180d0)
END IF

asce2rad = deg2rad(asce2)
epsrad = deg2rad(eps)

IF ((asce2>=0d0).AND.(asce2<=90d0)) THEN
    Decli = DATAN(DSIN(asce2rad)*DTAN(epsrad))
ELSEIF (asce2>90d0) THEN
    Decli = DATAN(DSIN(asce2rad-pi)*DTAN(epsrad))
ELSEIF ((asce2<=0).AND.(asce2>=-90)) THEN
    Decli = DATAN(DSIN(DABS(asce2rad))*DTAN(epsrad))
ELSEIF (asce2<-90) THEN
    Decli = DATAN(DSIN(DABS(asce2rad-pi))*DTAN(epsrad))
END IF

Decli = rad2deg(Decli)

END FUNCTION Decli
! ============================================================

END MODULE AsceDecl_class
!*****