!****h* TrasfMatrx/Globals_class
! NAME
! Globals_class -- Defines a set of global parameters and functions for dll file
! FUNCTION
! Contains a set of generic parameters and function that are used by other modules in dll
! AUTHOR
! Paolo Zoccarato
! CREATION DATE
! 2006/06/20
! MODIFICATION HISTORY
! +------------------------------------------------------------------+
! |  name      | date   |                  reason                    |
! |------------|--------|--------------------------------------------|
! |            |        |  No one at the moment                      |
! +------------|--------|--------------------------------------------+
! |            |        |                                            |
! +------------------------------------------------------------------+
! PARAMETERS
! PUBLIC CONSTANTS:
! pi      : pi
! one     : one in REAL(8) format
! zero    : zero in REAL(8) format
! lambda1 : GPS carrier L_1 wavelength [km]
! lambda2 : GPS carrier L_2 wavelength [km]
! zeros   : tridimensional vector of zeros in REAL(8) format
! COMMANDS
! PUBLIC FUNCTIONS:
!	norm2(vect) -- Calculates the norm 2 of the vector "vect"
!		INPUT PARAMETERS:
!			vect  : vector n-dimensional
!		OUTPUT PARAMETERS:
!			norm2 : 2 norme of the vector vect
!
!	deg2rad(D) -- Convert the angle "D" from degree to radiants
!		INPUT PARAMETERS:
!			D       : angle in degree
!		OUTPUT PARAMETERS:
!			deg2rad : angle D in radiants
!
!	rad2deg(R) -- Convert the angle "D" from radiants to degree
!		INPUT PARAMETERS:
!			R       : angle in radiants
!		OUTPUT PARAMETERS:
!			deg2rad : angle D in degree
!
!	identity_matrix(n) -- Create an "n" order identity matrix
!		INPUT PARAMETERS:
!			n               : order of the identity matrix
!		OUTPUT PARAMETERS:
!			identity_matrix : identity matrix created
!
! PUBLIC SUBROUTINES:
!	mjd2dat(mjd, iyear, imonth, iday, hour, min, sec) -- Converts a date from MJF format to Calendar format
!		INPUT PARAMETERS:
!			mjd    : Modified Julian Day
!		OUTPUT PARAMETERS:
!			iday   : day of the month (1 <= iday <= 31)
!			imonth : month of the year (1 <= imonth <= 12)
!			iyear  : year (e.g., 1987)
!			hour   : hour of the day (0 <= hour < 24)
!			min    : minute of the day (0 <= hour < 60)
!			sec    : second of the day (0.0 <= hour < 60.0)
! NOTES
! Develloped with Compaq Visual Fortran for Windows on Windows XP
!*****

MODULE Globals_class

! Modulo che contiene alcune costanti e funzioni utilizzate che hanno valenza universale
!
! PUBLIC CONSTANTS:
! pi      : pi greco
! one     : uno in formato REAL(8)
! zero    : zero in formato REAL(8)
! lambda1 : lunghezza d'onda della portante L_1 [km]
! lambda2 : lunghezza d'onda della portante L_2 [km]
! zeros   : vettore tridimenzionale di zeri in formato REAL(8)
!
! PUBLIC FUNCTIONS:
! norm2(vect)
! rms(vect)
! deg2rad(D)
! rad2deg(R)
! identity_matrix(n)

IMPLICIT NONE

! ============================================================
! Dichiarazione delle costanti
REAL(8), PARAMETER :: pi = 3.14159265358979d0
REAL(8), PARAMETER :: lambda1 = 0.1902936727983649d0 ! m
REAL(8), PARAMETER :: lambda2 = 0.2442102134245683d0 ! m
INTEGER, SAVE :: dimXc, dimfile, dimRed, dimX, erase, count
REAL(8), SAVE :: A, omega, phi, dl0, l0, dth0, th0, dps0, ps0, fb1, fb2, fb3, lam
LOGICAL, SAVE :: brute, tunnel
INTEGER, ALLOCATABLE, SAVE :: active(:)
REAL(8), ALLOCATABLE, SAVE :: Epoch(:), Sun(:,:), Soy(:,:), GPS1(:,:), GPS2(:,:), Res(:,:), Xs(:), lamm(:), minsloc(:,:),&
                              XLB(:), XUB(:)
! ============================================================

CONTAINS

! ============================================================
REAL(8) FUNCTION norm2(vect)

! Funzione che calcola la norma 2 di un vettore
!
! PARAMETRI DI INPUT:
! vect  : vettore n dimensionale
!
! PARAMETRI DI OUTPUT:
! norm2 : norma 2 del vettore vect

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: vect(:)

! Inizio procedimento
norm2 = SQRT(SUM(vect**2.d0))

END FUNCTION norm2
! ============================================================

! ============================================================
REAL(8) FUNCTION rms(vect)

! Funzione che calcola la rms di un vettore
!
! PARAMETRI DI INPUT:
! vect  : vettore n dimensionale
!
! PARAMETRI DI OUTPUT:
! rms : norma 2 del vettore vect

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: vect(:)

! Inizio procedimento
rms = SQRT(SUM(vect**2.d0)/SIZE(vect))

END FUNCTION rms
! ============================================================

! ============================================================
REAL(8) FUNCTION deg2rad(D)

! Funzione che converte l'input da gradi a radianti
!
! PARAMETRI DI INPUT:
! D       : angolo in gradi
!
! PARAMETRI DI OUTPUT:
! deg2rad : angolo D in radianti

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: D

! Inizio procedimento
deg2rad = D*pi/180.d0

END FUNCTION deg2rad
! ============================================================

! ============================================================
REAL(8) FUNCTION rad2deg(R)

! Funzione che converte l'input da radianti a gradi
!
! PARAMETRI DI INPUT:
! R       : angolo in radianti
!
! PARAMETRI DI OUTPUT:
! rad2deg : angolo R in gradi

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: R

! Inizio procedimento
rad2deg = R*180.d0/pi

END FUNCTION rad2deg
! ============================================================

! ============================================================
FUNCTION identity_matrix(n)

! Funzione che crea la matrice identit di dimensione n
!
! PARAMETRI DI INPUT:
! n               : dimensione della matrice
!
! PARAMETRI DI OUTPUT:
! identity_matrix : matrice identit di dimensione n creata

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: n
REAL(8) :: identity_matrix(n,n)
INTEGER :: J

! Inizio procedimento
identity_matrix = 0.d0

DO j = 1, n
	identity_matrix(j, j) = 1.d0
END DO

END FUNCTION identity_matrix
! ============================================================

! ============================================================
SUBROUTINE mjd2dat(mjd, year, sec)

! Subroutine che converte una data dal formato MJD al formato Calendar
! PARAMETRI DI DI INPUT:
! mjd  : Modified Julian Day
!
! PARAMETRI DI OUTPUT:
! year : year with month
! sec  : second of the day

IMPLICIT NONE 

! Variables
REAL(8), INTENT(IN) :: mjd
REAL(8), INTENT(OUT) :: year, sec
INTEGER :: iyear, imonth, iday, hour, min, ia, ib, ic, id, ie, ig
REAL(8) :: a, hours, mins

! Inizio computazione
a = mjd+2400001.d0+33282.d0
ia = a
hours = (a-ia)*24.d0
IF (ia.LT.2299161.d0) THEN 
	ic = ia+1524.d0
ELSE
	ib = FLOOR((ia-1867216.25d0)/36524.25d0)
	ic = ia+ib-FLOOR(ib/4.d0)+1525.d0
END IF
id = FLOOR((ic-122.1d0)/365.25d0)
ie = FLOOR(365.25d0*id)
ig = FLOOR((ic-ie)/30.6001d0)
iday = ic-ie-FLOOR(30.6001d0*ig)
imonth = ig-1.d0-12.d0*FLOOR(ig/14.d0)
iyear = id-4715.d0-FLOOR((7.d0+imonth)/10.d0)
hour = FLOOR(ABS(hours))
mins = (ABS(hours)-hour)*60.d0
min = FLOOR(mins)
sec = (mins-min)*60.d0+min*60.d0+hour*24.d0
year = iyear+imonth/12.d0

END SUBROUTINE mjd2dat
! ============================================================

! ============================================================
SUBROUTINE AssignX(X)

! Subroutine che crea il vettore di stato X
!
! PARAMETRI DI OUTPUT:
! X

IMPLICIT NONE 

! Variables
REAL(8), INTENT(OUT) :: X(dimX)
INTEGER :: i, j
LOGICAL :: finded(dimXc+1)
REAL(8) :: tmp(dimXc)

tmp = (/ A, omega, phi, dl0, l0, dth0, th0, dps0, ps0, fb1, fb2, fb3 /)
finded = .FALSE.
DO i = 1, dimX
	finded(dimXc+1) = .FALSE.
	DO j = 1, dimXc
		IF ((active(j).EQV..TRUE.).AND.(finded(j).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
			X(i) = tmp(j)
			finded(j) = .TRUE.
			finded(dimXc+1) = .TRUE.
		END IF
		IF (finded(dimXc+1).EQV..TRUE.) EXIT
	END DO
END DO

END SUBROUTINE AssignX
! ============================================================

! ============================================================
SUBROUTINE AssignB(bndL, bndU, XLB, XUB)

! Subroutine che crea i limiti del vettore di stato X
! PARAMETRI DI INPUT:
! bndL
! bndU
!
! PARAMETRI DI OUTPUT:
! XLB
! XUB

IMPLICIT NONE 

! Variables
REAL(8), INTENT(IN) :: bndL(dimXc), bndU(dimXc)
REAL(8), INTENT(OUT) :: XLB(dimX), XUB(dimX)
INTEGER :: i, j
LOGICAL :: finded(dimXc+1)

finded = .FALSE.
DO i = 1, dimX
	finded(dimXc+1) = .FALSE.
	DO j = 1, dimXc
		IF ((active(j).EQV..TRUE.).AND.(finded(j).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
			XLB(i) = bndL(j)
			XUB(i) = bndU(j)
			finded(j) = .TRUE.
			finded(dimXc+1) = .TRUE.
		END IF
		IF (finded(dimXc+1).EQV..TRUE.) EXIT
	END DO
END DO

END SUBROUTINE AssignB
! ============================================================

! ============================================================
SUBROUTINE AssignP(X)

! Sunbroutine che crea il vettore di stato X
! PARAMETRI DI DI INPUT:
! X

IMPLICIT NONE 

! Variables
REAL(8), INTENT(IN) :: X(dimX)
INTEGER :: i
LOGICAL :: finded(dimXc+1)

! Inizio procedimento
finded = .FALSE.
DO i = 1, dimX
	finded(dimXc+1) = .FALSE.
	IF ((active(1).EQV..TRUE.).AND.(finded(1).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		A = X(i)
		finded(1) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(2).EQV..TRUE.).AND.(finded(2).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		omega = X(i)
		finded(2) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(3).EQV..TRUE.).AND.(finded(3).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		phi = X(i)
		finded(3) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(4).EQV..TRUE.).AND.(finded(4).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		dl0 = X(i)
		finded(4) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(5).EQV..TRUE.).AND.(finded(5).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		l0 = X(i)
		finded(5) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(6).EQV..TRUE.).AND.(finded(6).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		dth0 = X(i)
		finded(6) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(7).EQV..TRUE.).AND.(finded(7).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		th0 = X(i)
		finded(7) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(8).EQV..TRUE.).AND.(finded(8).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		dps0 = X(i)
		finded(8) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(9).EQV..TRUE.).AND.(finded(9).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		ps0 = X(i)
		finded(9) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(10).EQV..TRUE.).AND.(finded(10).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		fb1 = X(i)
		finded(10) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(11).EQV..TRUE.).AND.(finded(11).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		fb2 = X(i)
		finded(11) = .TRUE.
		finded(dimXc+1) = .TRUE.
	ELSEIF ((active(12).EQV..TRUE.).AND.(finded(12).EQV..FALSE.).AND.(finded(dimXc+1).EQV..FALSE.)) THEN
		fb3 = X(i)
		finded(12) = .TRUE.
		finded(dimXc+1) = .TRUE.
	END IF
END DO

END SUBROUTINE AssignP
! ============================================================

END MODULE Globals_class
!*****