MODULE ReadFiles_class

USE Globals_class, ONLY : dimfile, dimRed, Epoch, Sun, Soy, GPS1, GPS2, Res, active, AssignX

! Modulo per la lettura degli istanti, dei residui, delle posizioni del Sole,
! della Soyuz e dei satellii GPS da file
!
! PUBLIC INTERFACES:
! Count
! ReadFile

IMPLICIT NONE

PRIVATE :: Reading, CountRows, ReadDat, GoodRows, ReduceDat, selectDat

! ============================================================
INTERFACE ReadFile

MODULE PROCEDURE Reading

END INTERFACE ReadFile
! ============================================================

CONTAINS

! ============================================================
SUBROUTINE Reading(filename)

! Subroutine ausiliaria che riporta il numero di righe di cui  composto un file
!
! PARAMETRI DI INPUT:
! filename : nome del file contenente le posizioni del Sole (in [Km])
!            e i rispettivi tempi in formato MGD
!
! PARAMETRI DI OUTPUT:
! nrows    : numero di righe da cui  costituito il file "filename"

IMPLICIT NONE

! Variables
CHARACTER(LEN=*), INTENT(IN) :: filename
REAL(8), ALLOCATABLE :: Epocht(:), Sunt(:,:), Soyt(:,:), GPS1t(:,:), GPS2t(:,:), Rest(:,:)

! Inizio procedimento
IF (ALLOCATED(Epoch)) DEALLOCATE(Epoch)
IF (ALLOCATED(Sun))	DEALLOCATE(Sun)
IF (ALLOCATED(Soy))	DEALLOCATE(Soy)
IF (ALLOCATED(GPS1)) DEALLOCATE(GPS1)
IF (ALLOCATED(GPS2)) DEALLOCATE(GPS2)
IF (ALLOCATED(Res))	DEALLOCATE(Res)

CALL CountRows(dimfile, filename)

!WRITE(8,101) filename, dimfile
!101 FORMAT("Il file ", A, " contiene ", I5, " righe.")

ALLOCATE(Epocht(dimfile))
ALLOCATE(Sunt(dimfile, 3))
ALLOCATE(Soyt(dimfile, 6))
ALLOCATE(GPS1t(dimfile, 7))
ALLOCATE(GPS2t(dimfile, 7))
ALLOCATE(Rest(dimfile, 3))

CALL ReadDat(Epocht, Sunt, Soyt, GPS1t, GPS2t, Rest, filename)

CALL GoodRows(dimRed, Epocht, Rest)

ALLOCATE(Epoch(dimRed))
ALLOCATE(Sun(dimRed, 3))
ALLOCATE(Soy(dimRed, 6))
ALLOCATE(GPS1(dimRed, 7))
ALLOCATE(GPS2(dimRed, 7))
ALLOCATE(Res(dimRed, 3))

CALL ReduceDat(Epoch, Sun, Soy, GPS1, GPS2, Res, Epocht, Sunt, Soyt, GPS1t, GPS2t, Rest)

DEALLOCATE(Sunt)
DEALLOCATE(Soyt)
DEALLOCATE(GPS1t)
DEALLOCATE(GPS2t)
DEALLOCATE(Rest)

END SUBROUTINE Reading
! ============================================================

! ============================================================
SUBROUTINE CountRows(nrows, filename)

! Subroutine ausiliaria che riporta il numero di righe di cui  composto un file
!
! PARAMETRI DI INPUT:
! filename : nome del file contenente le posizioni del Sole (in [Km])
!            e i rispettivi tempi in formato MGD
!
! PARAMETRI DI OUTPUT:
! nrows    : numero di righe da cui  costituito il file "filename"

IMPLICIT NONE

! Variables
CHARACTER(len=*), INTENT(IN) :: filename
INTEGER, PARAMETER :: in=3.d0
INTEGER :: ios, ierr
INTEGER, INTENT(OUT) :: nrows
REAL :: x

! Inizio procedimento
nrows=0
OPEN(UNIT=in, FILE=filename, IOSTAT=ios)!, POSITION="REWIND", ACTION="READ")
IF (ios == 0) THEN
	DO
		READ(in, FMT='(A)', IOSTAT=ierr), x
		IF(ierr/=0) EXIT
		nrows = nrows + 1.d0
	END DO
ELSE
	DO
		PRINT *, "Unable to open file ", filename
		PAUSE
		STOP
	END DO
END IF
CLOSE(in)

END SUBROUTINE CountRows
! ============================================================

! ============================================================
SUBROUTINE ReadDat(REpoch, RSun, RSoy, RGPS1, RGPS2, RRes, filename)

! Subroutine ausiliaria che legge la data (TIA in formato MJD),
! le posizioni dei satelliti GPS [km-km/s], della Soyuz [km-km/s],
! del Sole [km] e i residui di fase dal file "filename"
!
! PARAMETRI DI INPUT:
! filename : nome del file contenente i dati
!
! PARAMETRI DI OUTPUT:
!          : matrice contenente nella prima colonna i tempi MGD delle
!            posizioni del Sole lette dal file in input e nelle
!            successive tre colonne la posizione (x, y, z) in [Km] del Sole

IMPLICIT NONE

! Variables
CHARACTER(LEN=*), INTENT(IN) :: filename
REAL(8), INTENT(OUT) :: REpoch(dimfile), RSun(dimfile, 3), RSoy(dimfile, 6),&
						RGPS1(dimfile, 7), RGPS2(dimfile, 7), RRes(dimfile, 3)
INTEGER, PARAMETER :: in = 10.d0
INTEGER :: i, ios, ierr, nsat1, nsat2, conf, nmis
REAL(8) :: mjd, GPS1Stat(6), GPS2Stat(6), SoyuzStat(6), ResPh, SunPos(4)

! Inizio Procedimento
OPEN(UNIT=in, FILE=filename, POSITION="REWIND", STATUS="OLD", ACTION="READ", IOSTAT=ios)
IF (ios == 0) THEN
	DO i = 1, dimfile
		READ(in, FMT=100, iostat=ierr) mjd, GPS1Stat, nsat1,&
			 GPS2Stat, nsat2, SoyuzStat, ResPh, conf, nmis, SunPos
		IF (ierr == 0) THEN
			REpoch(i) = mjd
			RSun(i, 1:3) = SunPos(1:3)
			RSoy(i, 1:6) = SoyuzStat
			RGPS1(i, 1) = nsat1
			RGPS1(i, 2:7) = GPS1Stat
			RGPS2(i, 1) = nsat2
			RGPS2(i, 2:7) = GPS2Stat
			RRes(i, 1:2) = (/ nsat1, nsat2 /)
			RRes(i, 3) = ResPh
		END IF
	ENDDO
ELSE
	PRINT *,"Unable to open file ", filename
	PAUSE
	STOP
END IF

CLOSE(in)

! Formati
100 FORMAT(F20.14,1X,2(6(F13.6,1X),I2,1X),6(F13.6,1X),E50.40,1X,I5,1X,I5,4(1X,F20.10))

END SUBROUTINE ReadDat
! ============================================================

! ============================================================
SUBROUTINE GoodRows(nrows, REpoch, RRes)

!
! PARAMETRI DI INPUT:
!
! PARAMETRI DI OUTPUT:
!

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: REpoch(dimfile), RRes(dimfile, 3)
INTEGER, INTENT(OUT) :: nrows
INTEGER :: i, j, insp, sn1_i, sn2_i, sn1_j, sn2_j
REAL(8) :: mjd_i, mjd_j, res_i, res_j, pres(dimfile)

! Inizio procedimento
insp = 0
pres = (/ (i*0.d0, i = 1, dimfile) /)
DO i = 1, dimfile
	IF (pres(i).EQ.0) THEN
		mjd_i = REpoch(i)
		sn1_i = RRes(i, 1)
		sn2_i = RRes(i, 2)
		res_i = RRes(i, 3)
		DO j = i+1, dimfile
			mjd_j = REpoch(j)
			sn1_j = RRes(j, 1)
			sn2_j = RRes(j, 2)
			res_j = RRes(j, 3)
			IF ((mjd_i.EQ.mjd_j).AND.(sn1_i.EQ.sn1_j).AND.(sn2_i.EQ.sn2_j)) THEN
				pres(j) = 1.d0
				insp = insp+1
			END IF
		END DO
	END IF
END DO
nrows = dimfile-insp

END SUBROUTINE GoodRows
! ============================================================

! ============================================================
SUBROUTINE ReduceDat(EpochMed, SunMed, SoyMed, GPS1Med, GPS2Med, ResMed,&
					 REpoch, RSun, RSoy, RGPS1, RGPS2, RRes)

!
! PARAMETRI DI INPUT:
!
! PARAMETRI DI OUTPUT:
!

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: REpoch(dimfile), RSun(dimfile, 3), RSoy(dimfile, 6),&
					   RGPS1(dimfile, 7), RGPS2(dimfile, 7), RRes(dimfile, 3)
REAL(8), INTENT(OUT) :: EpochMed(dimRed), SunMed(dimRed, 3), SoyMed(dimRed, 6),&
						GPS1Med(dimRed, 7), GPS2Med(dimRed, 7), ResMed(dimRed, 3)
INTEGER :: i, j, ins, sn1_i, sn2_i, sn1_j, sn2_j, pres(dimfile), count
REAL(8) :: mjd_i, mjd_j, resid, res_i, res_j

! Inizio Procedimento
pres = (/ (i*0.d0, i = 1, dimfile) /)
ins = 1;
DO i = 1, dimfile
	IF (pres(i).EQ.0) THEN
		mjd_i = REpoch(i)
		sn1_i = RRes(i, 1)
		sn2_i = RRes(i, 2)
		res_i = RRes(i, 3)
		resid = res_i
		count = 1
		DO j = i+1, dimfile
			mjd_j = REpoch(j)
			sn1_j = RRes(j, 1)
			sn2_j = RRes(j, 2)
			res_j = RRes(j, 3)
			IF ((mjd_i.EQ.mjd_j).AND.(sn1_i.EQ.sn1_j).AND.(sn2_i.EQ.sn2_j)) THEN
				resid = resid+res_j
				count = count+1
				pres(j) = 1
			END IF
		END DO
		EpochMed(ins) = REpoch(i)
		SunMed(ins, 1:3) = RSun(i, 1:3)
		SoyMed(ins, 1:6) = RSoy(i, 1:6)
		GPS1Med(ins, 1:7) = RGPS1(i, 1:7)
		GPS2Med(ins, 1:7) = RGPS2(i, 1:7)
		ResMed(ins, 1:3)= RRes(i, 1:3)
		ins = ins+1
	END IF
END DO

! Formati
100 FORMAT(F20.14,1X,2(6(F13.6,1X),I2,1X),6(F13.6,1X),E50.40,1X,I5,1X,I5,4(1X,F20.10))

END SUBROUTINE ReduceDat
! ============================================================

! ============================================================
SUBROUTINE selectDat(EpochMm, SunMm, SoyMm, GPS1Mm, GPS2Mm, ResMm,&
					 REpoch, RSun, RSoy, RGPS1, RGPS2, RRes, n)

!
! PARAMETRI DI INPUT:
!
! PARAMETRI DI OUTPUT:
!

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: REpoch(dimfile), RSun(dimfile, 3), RSoy(dimfile, 6),&
					   RGPS1(dimfile, 7), RGPS2(dimfile, 7), RRes(dimfile, 3)
INTEGER, INTENT(IN) :: n
REAL(8), INTENT(OUT) :: EpochMm(dimRed), SunMm(dimRed, 3), SoyMm(dimRed, 6),&
						GPS1Mm(dimRed, 7), GPS2Mm(dimRed, 7), ResMm(dimRed, 3)
INTEGER :: i, j, ins, sn1_i, sn2_i, sn1_j, sn2_j, pres(dimfile)
REAL(8) :: mjd_i, mjd_j, resid, res_i, res_j

! Inizio Procedimento
pres = (/ (i*0.d0, i = 1, dimfile) /)
ins = 1;
DO i = 1, dimfile
	IF (pres(i).EQ.0) THEN
		mjd_i = REpoch(i)
		sn1_i = RRes(i, 1)
		sn2_i = RRes(i, 2)
		res_i = RRes(i, 3)
		resid = res_i
		DO j = i+1, dimfile
			mjd_j = REpoch(j)
			sn1_j = RRes(j, 1)
			sn2_j = RRes(j, 2)
			res_j = RRes(j, 3)
			IF (n.EQ.0) THEN
				IF ((mjd_i.EQ.mjd_j).AND.(sn1_i.EQ.sn1_j).AND.(sn2_i.EQ.sn2_j)) THEN
					pres(j) = 1
					IF (res_j<resid) resid = res_j
				END IF
			ELSEIF (n.EQ.1) THEN
				IF ((mjd_i.EQ.mjd_j).AND.(sn1_i.EQ.sn1_j).AND.(sn2_i.EQ.sn2_j)) THEN
					pres(j) = 1
					IF (res_j>resid) resid = res_j
				END IF
			ELSE
				WRITE(8,*) 'Wrong method selection: must be 0 or 1!'
				PRINT *, 'Wrong method selection: must be 0 or 1!'
			END IF
		END DO
		EpochMm(ins) = REpoch(i)
		SunMm(ins, 1:3) = RSun(i, 1:3)
		SoyMm(ins, 1:6) = RSoy(i, 1:6)
		GPS1Mm(ins, 1:7) = RGPS1(i, 1:7)
		GPS2Mm(ins, 1:7) = RGPS2(i, 1:7)
		ResMm(ins, 1:3)= RRes(i, 1:3)
		ins = ins+1
	END IF
END DO

! Formati
100 FORMAT(F20.14,1X,2(6(F13.6,1X),I2,1X),6(F13.6,1X),E50.40,1X,I5,1X,I5,4(1X,F20.10))

END SUBROUTINE selectDat
! ============================================================

END MODULE ReadFiles_class