MODULE Transform_class

USE RotoTrasl_class

! Modulo che calcola le matrcici di trasformazione tra i vari sistemi di riferiemnto I, S, P, B
!
! PUBLIC FUNCTIONS:
! RSI(alpha, delta, gamma, p_I, p_S)
! TSI(alpha, delta, gamma, P_I_CM, p_I, p_S)
! TIS(alpha, delta, gamma, P_I_CM, p_S, p_I)
! RPS(lambda, theta, p_S, p_P)
! TPS(lambda, theta, p_S, p_P)
! TSP(lambda, theta, p_P, p_S)
! RBP(psi, p_P, p_B)
! TBP(psi, p_P, p_B)
! TPB(psi, p_B, p_P)
!
! PUBLIC SUBROUTINES:
! RBS(lambda, theta, psi, p_S, p_B)
! TBS(lambda, theta, psi, p_S, p_B)
! TSB(lambda, theta, psi, p_B, p_S)
! RPI(alpha, delta, gamma, lambda, theta, p_I, p_P)
! TPI(alpha, delta, gamma, lambda, theta, P_I_CM, p_I, p_P)
! TIP(alpha, delta, gamma, lambda, theta, P_I_CM, p_P, p_I)
! RBI(alpha, delta, gamma, lambda, theta, psi, p_B, p_I)
! TBI(alpha, delta, gamma, lambda, theta, psi, P_I_CM, p_B, p_I)
! TIB(alpha, delta, gamma, lambda, theta, psi, P_I_CM, p_I, p_B)

IMPLICIT NONE

CONTAINS

! ============================================================
FUNCTION RSI(alpha, delta, gamma)

! Funzione che calcola la matrice di rotazione dal sdr S al sdr I
!
! PARAMETRI DI INPUT
! alpha : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta : declinazione del versore s2 rispetto al sdr I [deg]
! gamma : declinazione del versore s1 rispetto al sdr I [deg]
!
! PARAMETRI DI OUTPUT:
! RSI   : matrice di trasformazione dal sdr S al sdr I

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma
REAL(8) :: RSI(3,3)

! Inizio procedimento
RSI = MATMUL(MATMUL(rot(-90.d0+alpha, 0.d0, 0.d0), rot(0.d0, 0.d0, -delta)), rot(0.d0, gamma, 0.d0))

END FUNCTION RSI
! ============================================================

! ============================================================
FUNCTION TSI(alpha, delta, gamma, P_I_CM)

! Funzione che calcola la matrice di trasformazione dal sdr S al sdr I
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! gamma : declinazione del versore s1 rispetto al sdr I [deg]
! P_I_CM : posizione del CM nel sdr I [Km]
!
! PARAMETRI DI OUTPUT:
! TSI    : matrice di trasformazione dal sdr S al sdr I

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma, P_I_CM(3)
REAL(8) :: TSI(4,4)

! Inizio procedimento
TSI = trasm(RSI(alpha, delta, gamma), P_I_CM)

END FUNCTION TSI
! ============================================================

! ============================================================
FUNCTION TIS(alpha, delta, gamma, P_I_CM)

! Funzione che calcola la matrice di trasformazione dal sdr I al sdr S
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! gamma : declinazione del versore s1 rispetto al sdr I [deg]
! P_I_CM : posizione del CM nel sdr I [Km]
!
! PARAMETRI DI OUTPUT:
! TIS    : matrice di trasformazione dal sdr I al sdr S

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma, P_I_CM(3)
REAL(8) :: TIS(4,4)

! Inizio procedimento
TIS = trasm(TRANSPOSE(RSI(alpha, delta, gamma)), -MATMUL(TRANSPOSE(RSI(alpha, delta, gamma)), P_I_CM))

END FUNCTION TIS
! ============================================================

! ============================================================
FUNCTION RPS(lambda, theta)

! Funzione che calcola la matrice di rotazione dal sdr P al sdr S
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
!
! PARAMETRI DI OUTPUT:
! RPS    : matrice di trasformazione dal sdr P al sdr S

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta
REAL(8) :: RPS(3,3)

! Inizio procedimento
RPS = MATMUL(rot(0.d0,theta,0.d0), rot(0.d0,0.d0,lambda))

END FUNCTION RPS
! ============================================================

! ============================================================
FUNCTION TPS(lambda, theta)

! Funzione che calcola la matrice di trasformazione dal sdr P al sdr S
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
!
! PARAMETRI DI OUTPUT:
! TPS    : matrice di trasformazione dal sdr P al sdr S

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta
REAL(8) :: TPS(4,4), P_O_P(3)

! Inizio procedimento
P_O_P = (/ 0.d0, 0.d0, 0.d0 /)
TPS = trasm(RPS(lambda, theta), P_O_P)

END FUNCTION TPS
! ============================================================

! ============================================================
FUNCTION TSP(lambda, theta)

! Funzione che calcola la matrice di trasformazione dal sdr S al sdr P
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
!
! PARAMETRI DI OUTPUT:
! TSP    : matrice di trasformazione dal sdr S al sdr P

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta
REAL(8) :: TSP(4,4), P_O_P(3)

! Inizio procedimento
P_O_P = (/ 0.d0, 0.d0, 0.d0 /)
TSP = trasm(TRANSPOSE(RPS(lambda, theta)), -MATMUL(TRANSPOSE(RPS(lambda, theta)), P_O_P))

END FUNCTION TSP
! ============================================================

! ============================================================
FUNCTION RBP(psi)

! Funzione che calcola la matrice di rotazione dal sdr B al sdr P
!
! PARAMETRI DI INPUT
! psi : angolo di spin [deg]
!
! PARAMETRI DI OUTPUT:
! RBP : matrice di trasformazione dal sdr B al sdr P

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: psi
REAL(8) :: RBP(3,3)

! Inizio procedimento
RBP = rot(0.d0, psi, 0.d0)

END FUNCTION RBP
! ============================================================

! ============================================================
FUNCTION TBP(psi)

! Funzione che calcola la matrice di trasformazione dal sdr B al sdr P
!
! PARAMETRI DI INPUT
! psi : angolo di spin [deg]
!
! PARAMETRI DI OUTPUT:
! TBP : matrice di trasformazione dal sdr B al sdr P

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: psi
REAL(8) :: TBP(4,4), P_O_B(3)

! Inizio procedimento
P_O_B = (/ 0.d0, 0.d0, 0.d0 /)
TBP = trasm(RBP(psi), P_O_B)

END FUNCTION TBP
! ============================================================

! ============================================================
FUNCTION TPB(psi)

! Funzione che calcola la matrice di trasformazione dal sdr P al sdr B
!
! PARAMETRI DI INPUT
! psi : angolo di spin [deg]
!
! PARAMETRI DI OUTPUT:
! TPB : matrice di trasformazione dal sdr P al sdr B

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: psi
REAL(8) :: TPB(4,4), P_O_B(3)

! Inizio procedimento
P_O_B = (/ 0.d0, 0.d0, 0.d0 /)
TPB = trasm(TRANSPOSE(RBP(psi)), -MATMUL(TRANSPOSE(RBP(psi)), P_O_B))

END FUNCTION TPB
! ============================================================

! ============================================================
SUBROUTINE RBS(lambda, theta, psi, p_B, p_S)

! Funzione che calcola la matrice di rotazione dal sdr B al sdr P
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! psi    : angolo di spin [deg]
! p_B    : vettore iniziale nel sdr B
!
! PARAMETRI DI OUTPUT:
! p_S    : vettore ruotato nel sdr S

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta, psi, p_B(3)
REAL(8), INTENT(OUT) :: p_S(3)

! Inizio procedimento
p_S = MATMUL(RPS(lambda, theta), MATMUL(RBP(psi), p_B))

END SUBROUTINE RBS
! ============================================================

! ============================================================
SUBROUTINE TBS(lambda, theta, psi, p_B, p_S)

! Funzione che calcola la matrice di trasformazione dal sdr B al sdr S
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! psi    : angolo di spin [deg]
! p_B    : vettore iniziale nel sdr B
!
! PARAMETRI DI OUTPUT:
! p_S    : vettore roto-traslato nel sdr S

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta, psi, p_B(3)
REAL(8), INTENT(OUT) :: p_S(3)
REAL(8) :: p_S_tmp(4)

! Inizio procedimento
p_S_tmp = MATMUL(TPS(lambda, theta), MATMUL(TBP(psi), (/ p_B, 1.d0 /)))
p_S = p_S_tmp(1:3)

END SUBROUTINE TBS
! ============================================================

! ============================================================
SUBROUTINE TSB(lambda, theta, psi, p_B, p_S)

! Funzione che calcola la matrice di trasformazione dal sdr I al sdr P
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! psi    : angolo di spin [deg]
! p_S    : vettore iniziale nel sdr S
!
! PARAMETRI DI OUTPUT:
! p_B    : vettore roto-traslato nel sdr B

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta, psi, p_S(3)
REAL(8), INTENT(OUT) :: p_B(3)
REAL(8) :: p_B_tmp(4)

! Inizio procedimento
p_B_tmp = MATMUL(TPB(psi), MATMUL(TSP(lambda, theta), (/ p_S, 1.d0 /)))
p_B = p_B_tmp(1:3)

END SUBROUTINE TSB
! ============================================================

! ============================================================
SUBROUTINE RPI(alpha, delta, gamma, lambda, theta, p_P, p_I)

! Funzione che calcola la matrice di rotazione dal sdr P al sdr I
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! gamma  : declinazione del versore s1 rispetto al sdr I [deg]
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! p_P    : vettore iniziale nel sdr P
!
! PARAMETRI DI OUTPUT:
! p_I    : vettore ruotato nel sdr I

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma, lambda, theta, p_P(3)
REAL(8), INTENT(OUT) :: p_I(3)

! Inizio procedimento
p_I = MATMUL(RSI(alpha, delta, gamma), MATMUL(RPS(lambda, theta), p_P))

END SUBROUTINE RPI
! ============================================================

! ============================================================
SUBROUTINE TPI(alpha, delta, gamma, lambda, theta, P_I_CM, p_P, p_I)

! Funzione che calcola la matrice di trasformazione dal sdr P al sdr I
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! gamma  : declinazione del versore s1 rispetto al sdr I [deg]
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! P_I_CM : posizione del CM nel sdr I [Km]
! p_P    : vettore iniziale nel sdr P
!
! PARAMETRI DI OUTPUT:
! p_I    : vettore roto-traslato nel sdr I

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma, lambda, theta, P_I_CM(3), p_P(3)
REAL(8), INTENT(OUT) :: p_I(3)
REAL(8) :: p_I_tmp(4)

! Inizio procedimento
p_I_tmp = MATMUL(TSI(alpha, delta, gamma, P_I_CM), MATMUL(TPS(lambda, theta), (/ p_P, 1.d0 /)))
p_I = p_I_tmp(1:3)

END SUBROUTINE TPI
! ============================================================

! ============================================================
SUBROUTINE TIP(alpha, delta, gamma, lambda, theta, P_I_CM, p_I, p_P)

! Funzione che calcola la matrice di trasformazione dal sdr I al sdr P
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! gamma  : declinazione del versore s1 rispetto al sdr I [deg]
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! P_I_CM : posizione del CM nel sdr I [Km]
! p_I    : vettore iniziale nel sdr I
!
! PARAMETRI DI OUTPUT:
! p_P    : vettore roto-traslato nel sdr P

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma, lambda, theta, P_I_CM(3), p_I(3)
REAL(8), INTENT(OUT) :: p_P(3)
REAL(8) :: p_P_tmp(4)

! Inizio procedimento
p_P_tmp = MATMUL(TSP(lambda, theta), MATMUL(TIS(alpha, delta, gamma, P_I_CM), (/ p_I, 1.d0 /)))
p_P = p_P_tmp(1:3)

END SUBROUTINE TIP
! ============================================================

! ============================================================
SUBROUTINE RBI(alpha, delta, gamma, lambda, theta, psi, p_B, p_I)

! Funzione che calcola la matrice di trasformazione dal sdr B al sdr I
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! gamma  : declinazione del versore s1 rispetto al sdr I [deg]
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! psi    : angolo di spin [deg]
! p_B    : vettore iniziale nel sdr B
!
! PARAMETRI DI OUTPUT:
! p_I    : vettore ruotato nel sdr I

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma, lambda, theta, psi, p_B(3)
REAL(8), INTENT(OUT) :: p_I(3)

! Inizio procedimento
p_I = MATMUL(RSI(alpha, delta, gamma), MATMUL(RPS(lambda, theta), MATMUL(RBP(psi), p_B)))

END SUBROUTINE RBI
! ============================================================

! ============================================================
SUBROUTINE TBI(alpha, delta, gamma, lambda, theta, psi, P_I_CM, p_B, p_I)

! Funzione che calcola la matrice di trasformazione dal sdr B al sdr I
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! gamma  : declinazione del versore s1 rispetto al sdr I [deg]
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! psi    : angolo di spin [deg]
! P_I_CM : posizione del CM nel sdr I [Km]
! p_B    : vettore iniziale nel sdr B
!
! PARAMETRI DI OUTPUT:
! p_I    : vettore roto-traslato nel sdr I

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma, lambda, theta, psi, P_I_CM(3), p_B(3)
REAL(8), INTENT(OUT) :: p_I(3)
REAL(8) :: p_I_tmp(4)

! Inizio procedimento
p_I_tmp = MATMUL(TSI(alpha, delta, gamma, P_I_CM), MATMUL(TPS(lambda, theta), MATMUL(TBP(psi), (/ p_B, 1.d0 /))))
p_I = p_I_tmp(1:3)

END SUBROUTINE TBI
! ============================================================

! ============================================================
SUBROUTINE TIB(alpha, delta, gamma, lambda, theta, psi, P_I_CM, p_I, p_B)

! Funzione che calcola la matrice di trasformazione dal sdr I al sdr B
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! gamma  : declinazione del versore s1 rispetto al sdr I [deg]
! lambda : angolo dell'ampiezza della precessione [deg]
! theta  : angolo di rotazione della precessione [deg]
! psi    : angolo di spin [deg]
! P_I_CM : posizione del CM nel sdr I [Km]
! p_I    : vettore iniziale nel sdr I
!
! PARAMETRI DI OUTPUT:
! p_B    : vettore roto-traslato nel sdr B

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, gamma, lambda, theta, psi, P_I_CM(3), p_I(3)
REAL(8), INTENT(OUT) :: p_B(3)
REAL(8) :: p_B_tmp(4)

! Inizio procedimento
p_B_tmp = MATMUL(TPB(psi), MATMUL(TSP(lambda, theta), MATMUL(TIS(alpha, delta, gamma, P_I_CM), (/ p_I, 1.d0 /))))
p_B = p_B_tmp(1:3)

END SUBROUTINE TIB
! ============================================================

END MODULE Transform_class